@@ -197,16 +197,16 @@ class Request(base_wsgi.Request):
     @utils.cached_property
     def accept_version(self):
         accept_header = self.headers.get('ACCEPT', "")
-        accept_version_re = re.compile(".*?application/vnd.openstack.trove"
-                                       "(\+.+?)?;"
-                                       "version=(?P<version_no>\d+\.?\d*)")
+        accept_version_re = re.compile(r".*?application/vnd.openstack.trove"
+                                       r"(\+.+?)?;"
+                                       r"version=(?P<version_no>\d+\.?\d*)")
 
         match = accept_version_re.search(accept_header)
         return match.group("version_no") if match else None
 
     @utils.cached_property
     def url_version(self):
-        versioned_url_re = re.compile("/v(?P<version_no>\d+\.?\d*)")
+        versioned_url_re = re.compile(r"/v(?P<version_no>\d+\.?\d*)")
         match = versioned_url_re.search(self.path)
         return match.group("version_no") if match else None
 
@@ -301,7 +301,7 @@ class Resource(base_wsgi.Resource):
             # If action_result is not a Fault then there really was a
             # serialization error which we log. Otherwise return the Fault.
             if not isinstance(action_result, Fault):
-                LOG.exception(_("Unserializable result detected."))
+                LOG.exception("Unserializable result detected.")
                 raise
             return action_result
 
@@ -322,6 +322,8 @@ class Controller(object):
             exception.BackupTooLarge,
             exception.ModuleAccessForbidden,
             exception.ModuleAppliedToInstance,
+            exception.PolicyNotAuthorized,
+            exception.LogAccessForbidden,
         ],
         webob.exc.HTTPBadRequest: [
             exception.InvalidModelError,
@@ -345,6 +347,7 @@ class Controller(object):
             exception.DatastoreNotFound,
             exception.SwiftNotFound,
             exception.ModuleTypeNotFound,
+            exception.RootHistoryNotFound,
         ],
         webob.exc.HTTPConflict: [
             exception.BackupNotCompleteError,
@@ -358,6 +361,7 @@ class Controller(object):
         webob.exc.HTTPServerError: [
             exception.VolumeCreationFailure,
             exception.UpdateGuestError,
+            exception.SwiftConnectionError,
         ],
         webob.exc.HTTPNotImplemented: [
             exception.VolumeNotSupported,
@@ -372,14 +376,13 @@ class Controller(object):
 
     @classmethod
     def get_schema(cls, action, body):
-        LOG.debug("Getting schema for %s:%s" %
-                  (cls.__class__.__name__, action))
+        LOG.debug("Getting schema for %(name)s:%(action)s",
+                  {'name': cls.__class__.__name__, 'action': action})
         if cls.schemas:
             matching_schema = cls.schemas.get(action, {})
             if matching_schema:
-                LOG.debug(
-                    "Found Schema: %s" % matching_schema.get("name",
-                                                             matching_schema))
+                LOG.debug("Found Schema: %s",
+                          matching_schema.get("name", matching_schema))
             return matching_schema
 
     @staticmethod
@@ -452,7 +455,7 @@ class Fault(webob.exc.HTTPException):
     code_wrapper = {
         400: webob.exc.HTTPBadRequest,
         401: webob.exc.HTTPUnauthorized,
-        403: webob.exc.HTTPUnauthorized,
+        403: webob.exc.HTTPForbidden,
         404: webob.exc.HTTPNotFound,
     }
 
@@ -548,13 +551,14 @@ class ContextMiddleware(base_wsgi.Middleware):
                                           is_admin=is_admin,
                                           limit=limits.get('limit'),
                                           marker=limits.get('marker'),
-                                          service_catalog=service_catalog)
+                                          service_catalog=service_catalog,
+                                          roles=roles)
         request.environ[CONTEXT_KEY] = context
 
     @classmethod
     def factory(cls, global_config, **local_config):
         def _factory(app):
-            LOG.debug("Created context middleware with config: %s" %
+            LOG.debug("Created context middleware with config: %s",
                       local_config)
             return cls(app)
 
@@ -576,7 +580,7 @@ class FaultWrapper(base_wsgi.Middleware):
                 return resp
             return resp
         except Exception as ex:
-            LOG.exception(_("Caught error: %s."),
+            LOG.exception("Caught error: %s.",
                           encodeutils.exception_to_unicode(ex))
             exc = webob.exc.HTTPInternalServerError()
             return Fault(exc)
