@@ -21,11 +21,12 @@ from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
 from trove.common.remote import create_nova_client
+from trove.common import timeutils
 from trove.common import utils
 from trove.db import get_db_api
 from trove.db import models as dbmodels
 from trove.flavor.models import Flavor as flavor_model
-
+from trove.volume_type import models as volume_type_models
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -34,9 +35,9 @@ db_api = get_db_api()
 
 def persisted_models():
     return {
-        'datastore': DBDatastore,
+        'datastores': DBDatastore,
         'capabilities': DBCapabilities,
-        'datastore_version': DBDatastoreVersion,
+        'datastore_versions': DBDatastoreVersion,
         'capability_overrides': DBCapabilityOverrides,
         'datastore_version_metadata': DBDatastoreVersionMetadata
     }
@@ -44,30 +45,34 @@ def persisted_models():
 
 class DBDatastore(dbmodels.DatabaseModelBase):
 
-    _data_fields = ['id', 'name', 'default_version_id']
+    _data_fields = ['name', 'default_version_id']
+    _table_name = 'datastores'
 
 
 class DBCapabilities(dbmodels.DatabaseModelBase):
 
-    _data_fields = ['id', 'name', 'description', 'enabled']
+    _data_fields = ['name', 'description', 'enabled']
+    _table_name = 'capabilities'
 
 
 class DBCapabilityOverrides(dbmodels.DatabaseModelBase):
 
-    _data_fields = ['id', 'capability_id', 'datastore_version_id', 'enabled']
+    _data_fields = ['capability_id', 'datastore_version_id', 'enabled']
+    _table_name = 'capability_overrides'
 
 
 class DBDatastoreVersion(dbmodels.DatabaseModelBase):
 
-    _data_fields = ['id', 'datastore_id', 'name', 'manager', 'image_id',
-                    'packages', 'active']
+    _data_fields = ['datastore_id', 'name', 'image_id', 'packages',
+                    'active', 'manager']
+    _table_name = 'datastore_versions'
 
 
 class DBDatastoreVersionMetadata(dbmodels.DatabaseModelBase):
 
-    _data_fields = ['id', 'datastore_version_id', 'key', 'value',
+    _data_fields = ['datastore_version_id', 'key', 'value',
                     'created', 'deleted', 'deleted_at', 'updated_at']
-    preserve_on_delete = True
+    _table_name = 'datastore_version_metadata'
 
 
 class Capabilities(object):
@@ -134,7 +139,7 @@ class Capabilities(object):
 
         self.capabilities = [override(obj) for obj in capability_defaults]
 
-        LOG.debug('Capabilities for datastore %(ds_id)s: %(capabilities)s' %
+        LOG.debug('Capabilities for datastore %(ds_id)s: %(capabilities)s',
                   {'ds_id': self.datastore_version_id,
                    'capabilities': self.capabilities})
 
@@ -145,7 +150,7 @@ class Capabilities(object):
         defaults and overrides and provides the one structure that should be
         used as the interface to controlling capabilities per datastore.
 
-        :returns Capabilities:
+        :returns: Capabilities
         """
         self = cls(datastore_version_id)
         self._load()
@@ -166,7 +171,7 @@ class BaseCapability(object):
         """
         The capability's id
 
-        :returns str:
+        :returns: str
         """
         return self.db_info.id
 
@@ -175,7 +180,7 @@ class BaseCapability(object):
         """
         Is the capability/feature enabled?
 
-        :returns bool:
+        :returns: bool
         """
         return self.db_info.enabled
 
@@ -225,7 +230,7 @@ class CapabilityOverride(BaseCapability):
         """
         The name of the capability.
 
-        :returns str:
+        :returns: str
         """
         return self.parent_name
 
@@ -234,7 +239,7 @@ class CapabilityOverride(BaseCapability):
         """
         The description of the capability.
 
-        :returns str:
+        :returns: str
         """
         return self.parent_description
 
@@ -244,7 +249,7 @@ class CapabilityOverride(BaseCapability):
         Because capability overrides is an association table there are times
         where having the capability id is necessary.
 
-        :returns str:
+        :returns: str
         """
         return self.db_info.capability_id
 
@@ -253,7 +258,7 @@ class CapabilityOverride(BaseCapability):
         """
         Generates a CapabilityOverride object from the capability_override id.
 
-        :returns CapabilityOverride:
+        :returns: CapabilityOverride
         """
         try:
             return cls(DBCapabilityOverrides.find_by(
@@ -274,7 +279,7 @@ class CapabilityOverride(BaseCapability):
                                         override to.
         :param enabled:                 Set enabled to True or False
 
-        :returns CapabilityOverride:
+        :returns: CapabilityOverride
         """
 
         return CapabilityOverride(
@@ -291,7 +296,7 @@ class Capability(BaseCapability):
         """
         The Capability name
 
-        :returns str:
+        :returns: str
         """
         return self.db_info.name
 
@@ -300,7 +305,7 @@ class Capability(BaseCapability):
         """
         The Capability description
 
-        :returns str:
+        :returns: str
         """
         return self.db_info.description
 
@@ -310,7 +315,7 @@ class Capability(BaseCapability):
         Generates a Capability object by looking up the capability first by
         ID then by name.
 
-        :returns Capability:
+        :returns: Capability
         """
         try:
             return cls(DBCapabilities.find_by(id=capability_id_or_name))
@@ -326,7 +331,7 @@ class Capability(BaseCapability):
         """
         Creates a new capability.
 
-        :returns Capability:
+        :returns: Capability
         """
         return Capability(DBCapabilities.create(
             name=name, description=description, enabled=enabled))
@@ -519,6 +524,38 @@ def get_datastore_version(type=None, version=None, return_inactive=False):
     return (datastore, datastore_version)
 
 
+def get_datastore_or_version(datastore=None, datastore_version=None):
+    """
+    Validate that the specified datastore/version exists, and return the
+    corresponding ids.  This differs from 'get_datastore_version' in that
+    you don't need to specify both - specifying only a datastore will
+    return 'None' in the ds_ver field.  Raises DatastoreNoVersion if
+    you pass in a ds_ver without a ds.  Originally designed for module
+    management.
+
+    :param datastore:           Datastore name or id
+    :param datastore_version:   Version name or id
+    :return:                    Tuple of ds_id, ds_ver_id if found
+    """
+
+    datastore_id = None
+    datastore_version_id = None
+    if datastore:
+        if datastore_version:
+            ds, ds_ver = get_datastore_version(
+                type=datastore, version=datastore_version)
+            datastore_id = ds.id
+            datastore_version_id = ds_ver.id
+        else:
+            ds = Datastore.load(datastore)
+            datastore_id = ds.id
+    elif datastore_version:
+        # Cannot specify version without datastore.
+        raise exception.DatastoreNoVersion(
+            datastore=datastore, version=datastore_version)
+    return datastore_id, datastore_version_id
+
+
 def update_datastore(name, default_version):
     db_api.configure_db(CONF)
     try:
@@ -562,32 +599,55 @@ def update_datastore_version(datastore, name, manager, image_id, packages,
 
 
 class DatastoreVersionMetadata(object):
+    @classmethod
+    def _datastore_version_find(cls, datastore_name,
+                                datastore_version_name):
+        """
+        Helper to find a datastore version id for a given
+        datastore and datastore version name.
+        """
+        db_api.configure_db(CONF)
+        db_ds_record = DBDatastore.find_by(
+            name=datastore_name
+        )
+        db_dsv_record = DBDatastoreVersion.find_by(
+            datastore_id=db_ds_record.id,
+            name=datastore_version_name
+        )
+
+        return db_dsv_record.id
+
     @classmethod
     def _datastore_version_metadata_add(cls, datastore_name,
                                         datastore_version_name,
                                         datastore_version_id,
                                         key, value, exception_class):
-        """Create an entry in the Datastore Version Metadata table."""
-        # Do we have a mapping in the db?
-        # yes: and its deleted then modify the association
-        # yes: and its not deleted then error on create
-        # no: then just create the new association
+        """
+        Create a record of the specified key and value in the
+        metadata table.
+        """
+        # if an association does not exist, create a new one.
+        # if a deleted association exists, undelete it.
+        # if an un-deleted association exists, raise an exception.
+
         try:
             db_record = DBDatastoreVersionMetadata.find_by(
                 datastore_version_id=datastore_version_id,
                 key=key, value=value)
             if db_record.deleted == 1:
                 db_record.deleted = 0
-                db_record.updated_at = utils.utcnow()
+                db_record.updated_at = timeutils.utcnow()
                 db_record.save()
                 return
             else:
                 raise exception_class(
                     datastore=datastore_name,
                     datastore_version=datastore_version_name,
-                    flavor_id=value)
+                    id=value)
         except exception.NotFound:
             pass
+
+        # the record in the database only contains the datastore_verion_id
         DBDatastoreVersionMetadata.create(
             datastore_version_id=datastore_version_id,
             key=key, value=value)
@@ -595,8 +655,19 @@ class DatastoreVersionMetadata(object):
     @classmethod
     def _datastore_version_metadata_delete(cls, datastore_name,
                                            datastore_version_name,
-                                           datastore_version_id,
                                            key, value, exception_class):
+        """
+        Delete a record of the specified key and value in the
+        metadata table.
+        """
+        # if an association does not exist, raise an exception
+        # if a deleted association exists, raise an exception
+        # if an un-deleted association exists, delete it
+
+        datastore_version_id = cls._datastore_version_find(
+            datastore_name,
+            datastore_version_name)
+
         try:
             db_record = DBDatastoreVersionMetadata.find_by(
                 datastore_version_id=datastore_version_id,
@@ -608,26 +679,20 @@ class DatastoreVersionMetadata(object):
                 raise exception_class(
                     datastore=datastore_name,
                     datastore_version=datastore_version_name,
-                    flavor_id=value)
+                    id=value)
         except exception.ModelNotFoundError:
             raise exception_class(datastore=datastore_name,
                                   datastore_version=datastore_version_name,
-                                  flavor_id=value)
+                                  id=value)
 
     @classmethod
     def add_datastore_version_flavor_association(cls, datastore_name,
                                                  datastore_version_name,
                                                  flavor_ids):
-        db_api.configure_db(CONF)
-        db_ds_record = DBDatastore.find_by(
-            name=datastore_name
-        )
-        db_datastore_id = db_ds_record.id
-        db_dsv_record = DBDatastoreVersion.find_by(
-            datastore_id=db_datastore_id,
-            name=datastore_version_name
-        )
-        datastore_version_id = db_dsv_record.id
+        datastore_version_id = cls._datastore_version_find(
+            datastore_name,
+            datastore_version_name)
+
         for flavor_id in flavor_ids:
             cls._datastore_version_metadata_add(
                 datastore_name, datastore_version_name,
@@ -638,19 +703,8 @@ class DatastoreVersionMetadata(object):
     def delete_datastore_version_flavor_association(cls, datastore_name,
                                                     datastore_version_name,
                                                     flavor_id):
-        db_api.configure_db(CONF)
-        db_ds_record = DBDatastore.find_by(
-            name=datastore_name
-        )
-        db_datastore_id = db_ds_record.id
-        db_dsv_record = DBDatastoreVersion.find_by(
-            datastore_id=db_datastore_id,
-            name=datastore_version_name
-        )
-        datastore_version_id = db_dsv_record.id
         cls._datastore_version_metadata_delete(
-            datastore_name, datastore_version_name,
-            datastore_version_id, 'flavor', flavor_id,
+            datastore_name, datastore_version_name, 'flavor', flavor_id,
             exception.DatastoreFlavorAssociationNotFound)
 
     @classmethod
@@ -689,3 +743,138 @@ class DatastoreVersionMetadata(object):
         else:
             msg = _("Specify both the datastore and datastore_version_id.")
             raise exception.BadRequest(msg)
+
+    @classmethod
+    def add_datastore_version_volume_type_association(cls, datastore_name,
+                                                      datastore_version_name,
+                                                      volume_type_names):
+        datastore_version_id = cls._datastore_version_find(
+            datastore_name,
+            datastore_version_name)
+
+        # the database record will contain
+        # datastore_version_id, 'volume_type', volume_type_name
+        for volume_type_name in volume_type_names:
+            cls._datastore_version_metadata_add(
+                datastore_name, datastore_version_name,
+                datastore_version_id, 'volume_type', volume_type_name,
+                exception.DatastoreVolumeTypeAssociationAlreadyExists)
+
+    @classmethod
+    def delete_datastore_version_volume_type_association(
+            cls, datastore_name,
+            datastore_version_name,
+            volume_type_name):
+        cls._datastore_version_metadata_delete(
+            datastore_name, datastore_version_name, 'volume_type',
+            volume_type_name,
+            exception.DatastoreVolumeTypeAssociationNotFound)
+
+    @classmethod
+    def list_datastore_version_volume_type_associations(cls,
+                                                        datastore_version_id):
+        """
+        List the datastore associations for a given datastore version id as
+        found in datastore version metadata. Note that this may return an
+        empty set (if no associations are provided)
+        """
+        if datastore_version_id:
+            return DBDatastoreVersionMetadata.find_all(
+                datastore_version_id=datastore_version_id,
+                key='volume_type', deleted=False
+            )
+        else:
+            msg = _("Specify the datastore_version_id.")
+            raise exception.BadRequest(msg)
+
+    @classmethod
+    def list_datastore_volume_type_associations(cls,
+                                                datastore_name,
+                                                datastore_version_name):
+        """
+        List the datastore associations for a given datastore and version.
+        """
+        if datastore_name and datastore_version_name:
+            datastore_version_id = cls._datastore_version_find(
+                datastore_name, datastore_version_name)
+            return cls.list_datastore_version_volume_type_associations(
+                datastore_version_id)
+        else:
+            msg = _("Specify the datastore_name and datastore_version_name.")
+            raise exception.BadRequest(msg)
+
+    @classmethod
+    def datastore_volume_type_associations_exist(cls,
+                                                 datastore_name,
+                                                 datastore_version_name):
+        return cls.list_datastore_volume_type_associations(
+            datastore_name,
+            datastore_version_name).count() > 0
+
+    @classmethod
+    def allowed_datastore_version_volume_types(cls, context,
+                                               datastore_name,
+                                               datastore_version_name):
+        """
+        List all allowed volume types for a given datastore and
+        datastore version. If datastore version metadata is
+        provided, then the valid volume types in that list are
+        allowed. If datastore version metadata is not provided
+        then all volume types known to cinder are allowed.
+        """
+        if datastore_name and datastore_version_name:
+            # first obtain the list in the dsvmetadata
+            datastore_version_id = cls._datastore_version_find(
+                datastore_name, datastore_version_name)
+
+            metadata = cls.list_datastore_version_volume_type_associations(
+                datastore_version_id)
+
+            # then get the list of all volume types
+            all_volume_types = volume_type_models.VolumeTypes(context)
+
+            # if there's metadata: intersect,
+            # else, whatever cinder has.
+            if (metadata.count() != 0):
+                # the volume types from metadata first
+                ds_volume_types = tuple(f.value for f in metadata)
+
+                # Cinder volume type names are unique, intersect
+                allowed_volume_types = tuple(
+                    f for f in all_volume_types
+                    if ((f.name in ds_volume_types) or
+                        (f.id in ds_volume_types)))
+            else:
+                allowed_volume_types = tuple(all_volume_types)
+
+            return allowed_volume_types
+        else:
+            msg = _("Specify the datastore_name and datastore_version_name.")
+            raise exception.BadRequest(msg)
+
+    @classmethod
+    def validate_volume_type(cls, context, volume_type,
+                             datastore_name, datastore_version_name):
+        if cls.datastore_volume_type_associations_exist(
+                datastore_name, datastore_version_name):
+            allowed = cls.allowed_datastore_version_volume_types(
+                context, datastore_name, datastore_version_name)
+            if len(allowed) == 0:
+                raise exception.DatastoreVersionNoVolumeTypes(
+                    datastore=datastore_name,
+                    datastore_version=datastore_version_name)
+            if volume_type is None:
+                raise exception.DataStoreVersionVolumeTypeRequired(
+                    datastore=datastore_name,
+                    datastore_version=datastore_version_name)
+
+            allowed_names = tuple(f.name for f in allowed)
+            for n in allowed_names:
+                LOG.debug("Volume Type: %s is allowed for datastore "
+                          "%s, version %s." %
+                          (n, datastore_name, datastore_version_name))
+            if volume_type not in allowed_names:
+                raise exception.DatastoreVolumeTypeAssociationNotFound(
+                    datastore=datastore_name,
+                    version_id=datastore_version_name,
+                    id=volume_type)
